//****************************************************************************************
// Program: file.c
//
// Author : Daniel Buergin (Code von Roland Riegel)
//
// Beschreibung: Diverse Routinen zum Umgang mit SD-Karten
//
//****************************************************************************************
// Version: 0.1
//****************************************************************************************
// Aenderungen:
//
// 19-Jan-2009 bue Erste Version
//
//****************************************************************************************
//
// Copyrights: GPL
//
//***************************************************************************************

// Includes 

#include <string.h>
#include <avr/pgmspace.h>
#include "file.h" 
#include "uart.h"

uint8_t find_file_in_dir(struct fat_fs_struct* fs, struct fat_dir_struct* dd, const char* name, struct fat_dir_entry_struct* dir_entry)
{
    while(fat_read_dir(dd, dir_entry))
    {
        if(strcmp(dir_entry->long_name, name) == 0)
        {
            fat_reset_dir(dd);
            return 1;
        }
    }

    return 0;
}

struct fat_file_struct* open_file_in_dir(struct fat_fs_struct* fs, struct fat_dir_struct* dd, const char* name)
{
    struct fat_dir_entry_struct file_entry;
    if(!find_file_in_dir(fs, dd, name, &file_entry))
        return 0;

    return fat_open_file(fs, &file_entry);
}

uint8_t print_disk_info(const struct fat_fs_struct* fs)
{
    if(!fs)
        return 0;

    struct sd_raw_info disk_info;
    if(!sd_raw_get_info(&disk_info))
        return 0;

    uart_puts_P("manuf:  0x"); uart_putc_hex(disk_info.manufacturer); uart_putc('\n'); uart_putc('\r');
    uart_puts_P("oem:    "); uart_puts((char*) disk_info.oem); uart_putc('\n');uart_putc('\r');
    uart_puts_P("prod:   "); uart_puts((char*) disk_info.product); uart_putc('\n');uart_putc('\r');
    uart_puts_P("rev:    "); uart_putc_hex(disk_info.revision); uart_putc('\n');uart_putc('\r');
    uart_puts_P("serial: 0x"); uart_putdw_hex(disk_info.serial); uart_putc('\n');uart_putc('\r');
    uart_puts_P("date:   "); uart_putw_dec(disk_info.manufacturing_month); uart_putc('/');
                                uart_putw_dec(disk_info.manufacturing_year); uart_putc('\n');uart_putc('\r');
    uart_puts_P("size:   "); uart_putdw_dec(disk_info.capacity / 1024 / 1024); uart_puts_P("MB\n\r");
    uart_puts_P("copy:   "); uart_putw_dec(disk_info.flag_copy); uart_putc('\n');uart_putc('\r');
    uart_puts_P("wr.pr.: "); uart_putw_dec(disk_info.flag_write_protect_temp); uart_putc('/');
                                uart_putw_dec(disk_info.flag_write_protect); uart_putc('\n');uart_putc('\r');
    uart_puts_P("format: "); uart_putw_dec(disk_info.format); uart_putc('\n');uart_putc('\r');
    uart_puts_P("free:   "); uart_putdw_dec(fat_get_fs_free(fs)); uart_putc('/');
                                   uart_putdw_dec(fat_get_fs_size(fs)); uart_putc('\n');uart_putc('\r');

    return 1;
}

