#ifndef MAIN_H
#define MAIN_H

//=====================================================================================
// Program: Temperatur_Feuchte Logger mit SD Karte
//
// Author : Daniel Buergin
//
//=====================================================================================
// Headerfile fuer Mainprogramm
//=====================================================================================

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include <string.h>
#include <math.h>
#include <util/twi.h>

#define F_CPU 16000000L
#include <util/delay.h>

#if F_CPU == 16000000L
#define OCR_1MS 125
#elif F_CPU == 14745600L
#define OCR_1MS 115
#endif

// Serielle Schnittstelle

#define UART0_BAUD 9600

// Some special Characters

#define CR 13
#define LF 10
#define STX 2
#define ETX 3

#define MUE     0xe4    // Das Zeichen Mue, 1110|0100
#define GRAD    0xdf    // Das Gradzeichen, 1110|1111
#define STR     0x7c    // Laengstrich    , 0111|1100
#define BLOCK   0xff    // Schwarzer Block, 1111|1111

#define ButtonPort PORTA
#define ButtonDDR  DDRA
#define ButtonPIN  PINA
#define B1         4

#define REPEAT_MASK (1<<B1)
#define REPEAT_START    50              // after 500ms
#define REPEAT_NEXT     20              // every 200ms

// Definitionen fuer die Debounce-Routine von Peter Dannegger
// bit = 1: key pressed
uint8_t key_state;
// key long press and repeat
uint8_t key_rpt;
// key press detect
uint8_t key_press;
//
uint8_t get_key_press( uint8_t key_mask );

void setup_buttons(void);

//=====================================================================================
// Port Befehle
//   - CBI: Loescht ein Bit
//   - SBI: Setzt ein Bit 
//
// Usage: 
//
// #define xx_PORT_CLK    PORTB,PB1
// CBI(xx_PORT_CLK);
//
// oder direkt mit: 
//
// _CBI(PORTA,PA1);
//=====================================================================================

//   Set Portpin to 0
#define CBI(x) _CBI(x)
#define _CBI(x,y) x &= ~(1<<y)

//   Set Portpin to 1
#define SBI(x) _SBI(x)
#define _SBI(x,y) x |= 1<<y

// Displaybuffer fuer die Anzeige

char disp_buffer[80];

// Eigene Delay-Routine 
extern void my_delay_10ms(uint8_t t);

// SD Routinen
void open_logging(void);
void close_logging(void);
char sd_header[100];

// Logging Intervall Parameter in Sekunden
// Max = 1h
// Min = 15s

#define MAX_LOGGING_INTERVAL 3600
#define MIN_LOGGING_INTERVAL 60

// Timer Stuff
void init_timer0(void);
volatile uint32_t sec_timer;

#endif
