//=======================================================================================
// Driver for Maxim MAX7219 8 x 7-Segment Display (SPI)
//
// by D. Buergin (daniel.buergin@bluewin.ch)
//=======================================================================================
// Diese Version arbeitet mit dem RAW-Mode also ohne Decodierung der Zeichen
//=======================================================================================

#include "main.h"
#include "max7219.h"

//=======================================================================================
// Write a Byte to the MAX7219
//=======================================================================================

void max7219_WriteByte (unsigned char data_out)
{
	unsigned char loop, mask; //0x80 = 10000000
	for (loop=0,mask=0x80;loop<8;loop++, mask=mask>>1)
	{
		// Clock Line runternehmen

		CBI(MAX7219_PORT_CLK);

		// Das Bit rausschreiben

		if (data_out & mask)
		{
			SBI(MAX7219_PORT_DIN);
		}
		else
		{
			CBI(MAX7219_PORT_DIN);
		}
			
		// Die Clockline wieder raufnehmen

		SBI(MAX7219_PORT_CLK);
	}
	CBI(MAX7219_PORT_CLK);
}

//=======================================================================================
// Schreibt auf den MAX7219
// - Zeichen wird im MAX7319_Font Array umgewandelt in die entspr. Segment-Codierung
// - Der Dezimalpunkt wird gesetzt
//=======================================================================================

void max7219_WriteDP (unsigned char address, char data)
{
	unsigned char chr;

	chr = max7219_LookupCode((unsigned char)data);
	chr = chr | 0x80; 

	max7219_Write_cmd(address, chr);
}

//=======================================================================================
// Schreibt auf den MAX7219
// - Zeichen wird im MAX7319_Font Array umgewandelt in die entspr. Segment-Codierung
//=======================================================================================

void max7219_Write (unsigned char address, char data)
{
	unsigned char chr;
	chr = max7219_LookupCode((unsigned char)data);

	max7219_Write_cmd(address, chr);
}

//=======================================================================================
// Schreibt auf den MAX7219
//=======================================================================================

void max7219_Write_cmd (unsigned char address, unsigned char data)
{
	// Die zwei Byte schreiben
	
	max7219_WriteByte(address);
	max7219_WriteByte(data);

	// Nun die LOAD Line pulsen, damit uebernimmt der MAX7219 die
	// Daten

	SBI(MAX7219_PORT_LOAD);
	CBI(MAX7219_PORT_LOAD);
}

//=======================================================================================
// Init the MAX7219
//=======================================================================================

void max7219_Init (void)
{
	// Set all DDR Register to Output

	SBI(MAX7219_DDR_CLK);
	SBI(MAX7219_DDR_DIN);
	SBI(MAX7219_DDR_LOAD);

	max7219_Write_cmd(REG_SHUTDOWN, 0x01);  // Shutdown aus
	// max7219_Write(0x09, 0xff);  // Decode-Mode: alles auf Code B
	max7219_Write_cmd(REG_DECODE, 0x00);  // Decode-Mode: alles auf Code B
	max7219_Write_cmd(REG_INTENSITY, 0x00);  // Intensitt
	max7219_Write_cmd(REG_SCAN_LIMIT, 0x03);  // 8 digits
}

//=======================================================================================
// MAX7219_LookupCode()
//
// Description: Wandelt ein ASCII Zeichen in den entsprechenden 7-segment Kode um.
//=======================================================================================

unsigned char max7219_LookupCode (unsigned char character)
{
	unsigned char i;

	// Alle Zeichen im Array abgrasen
	for (i = 0; MAX7219_Font[i].ascii; i++)
	{
		// Schauen ob Zeichen vorhanden
		if (character == MAX7219_Font[i].ascii)
		{
			return MAX7219_Font[i].segs;
		}
	}
	// Wenn nicht, den Buchstaben "E" fuer Error ausgeben
	return 0x4f;
}
