#ifndef MAX7219_H
#define MAX7219_H

//=======================================================================================
// Driver for Maxim MAX7219 8 x 7-Segment Display (SPI)
//
// by D. Buergin (daniel.buergin@bluewin.ch)
//=======================================================================================
// Diese Library arbeitet mit dem "no-decode" Mode. Die Segmente werden also direkt
// angesprochen. Siehe Datenblatt des MAX7219
//=======================================================================================

// CLK Port of MAX7219

#define MAX7219_PORT_CLK	PORTA,PA7
#define MAX7219_DDR_CLK		DDRA,PA7

// DIN Port of MAX7219

#define MAX7219_PORT_DIN	PORTA,PA5
#define MAX7219_DDR_DIN		DDRA,PA5

// LOAD Port of MAX7219

#define MAX7219_PORT_LOAD	PORTA,PA6
#define MAX7219_DDR_LOAD	DDRA,PA6

#define REG_SCAN_LIMIT 0x0b
#define REG_DECODE     0x09
#define REG_INTENSITY  0x0a
#define REG_SHUTDOWN   0x0c

//=======================================================================================
// LED Segments:         a
//                     ----
//                   f|    |b
//                    |  g |
//                     ----
//                   e|    |c
//                    |    |
//                     ----  o dp
//                       d
//   Register bits:
//      bit:  7  6  5  4  3  2  1  0
//           dp  a  b  c  d  e  f  g
//=======================================================================================

static const struct {
	unsigned char   ascii;
	unsigned char   segs;
} MAX7219_Font[] = {
  {' ', 0x00},
  {'-', 0x01},
  {'_', 0x08},
  {'0', 0x7e},
  {'1', 0x30},
  {'2', 0x6d},
  {'3', 0x79},
  {'4', 0x33},
  {'5', 0x5b},
  {'6', 0x5f},
  {'7', 0x70},
  {'8', 0x7f},
  {'9', 0x7b},
  {'n', 0x15},
  {'A', 0x77},
  {'B', 0x1f},
  {'C', 0x4e},
  {'D', 0x3d},
  {'E', 0x4f},
  {'F', 0x47},
  {'H', 0x36},
  {'L', 0x0e},
  {'O', 0x7e},
  {'P', 0x67},
  {'U', 0x3e},
  {'.', 0x80},
  {'\0', 0x00}
};

void max7219_WriteByte (unsigned char data_out);
void max7219_Write (unsigned char address, char data);
void max7219_WriteDP (unsigned char address, char data);
void max7219_Write_cmd (unsigned char address, unsigned char data);
void max7219_Init (void);
unsigned char max7219_LookupCode (unsigned char data);

#endif
